@@ -1,4 +1,4 @@
-import os, re, math, operator
+import os, re, math
 import numpy as np
 from shapely.geometry import Polygon, LineString, box
 from lxml import etree as ET
@@ -75,11 +75,13 @@ class Contour:
         '''Allows use of != between multiple contours.'''
         return self.output() != other.output()
 # Helper Functions
-    def overlaps(self, other):
+    def overlaps(self, other, threshold=(1+2**(-17))):
         '''Return 0 if no overlap.
         For closed traces: return 1 if AoU/AoI < threshold, return AoU/AoI if not < threshold
         For open traces: return 0 if # pts differs or distance between parallel pts > threshold
                          return 1 otherwise'''
+        if self._shape == None:self.popshape()
+        if other._shape == None:other.popshape()
         # Check bounding box
         if (not self.box().intersects(other.box()) and
             not self.box().touches(other.box()) ):
@@ -87,7 +89,6 @@ class Contour:
         # Check if both same class of contours
         if self.closed != other.closed:
             return 0
-        threshold = (1+2**(-17))    
         # Closed contours
         if self.closed:
             AoU = self._shape.union( other._shape ).area
@@ -480,12 +481,12 @@ class Image:
         return float( node.get('brightness') )
 
 class rObject:
-    def __init__(self, name=None, series=None):
+    def __init__(self, name=None, series=None, verbose=False):
         self.name = name
         self.series = self.chkSeries(series)
         
-        self.protrusion = name[-2:] # Protrusion number
-        self.dendrite = name[0:3] # Parent dendrite number
+        self.protrusion = self.getProtNumber() # Protrusion number (stored as 'p##')
+        self.dendrite = self.getDendNumber() # Parent dendrite number (stored as 'd##')
         
         self.rType = self.getrType()
         self.data = {} # updated in makeSpecific
@@ -494,25 +495,52 @@ class rObject:
         self.count = self.series.getStartEndCount(self.name)[2]
         
         self.makeSpecific()
-        
+    
+        if verbose:
+            print   
+            print('=== rObject Created ===')
+            print('name: '+self.name)
+            print('den: '+self.dendrite)
+            try:print('prot: '+self.protrusion)
+            except:print('No prot')
+            try:print('type: '+self.rType)
+            except:print('No type')
+    
+    def getDendNumber(self):
+        dend = re.compile('d[0-9]{1,}')
+        try:
+            return self.name[0:dend.match(self.name).end()]
+        except:
+            return None
+    
+    def getProtNumber(self):
+        prot = re.compile('p[0-9]{1,}')
+        try:
+            return self.name[prot.search(self.name).start():prot.search(self.name).end()]
+        except:
+            return None
+            
     def makeSpecific(self):
         '''Creates unique data for this rObject (depends on type)'''
-        rType = self.rType.lower()
-        if rType == 'p': # Protrusion
-            importantData = ['start', 'end', 'count']
-            self.children = self.findChildren()
-        elif rType == 'c': # C
-            importantData = ['start', 'end', 'count']
-        elif 'cfa' in rType: # CFA
-            importantData = ['start', 'end', 'count', 'surface area', 'flat area']
-        elif 'endo' in rType: # Endosome
-            importantData = ['start', 'end', 'count']
-        elif rType[0:3] == 'ser': # SER
-            importantData = ['start', 'end', 'count']
-        elif rType[0:2] == 'sp': # Spine
-            importantData = ['start', 'end', 'count', 'surface area', 'flat area', 'volume']
-        elif rType[0:2] == 'ax': # Axon
-            importantData = ['start', 'end', 'count']
+        if self.rType != None:
+            rType = self.rType.lower()
+            if rType == 'p': # Protrusion
+                importantData = ['start', 'end', 'count']
+                self.children = self.findChildren()
+            elif rType == 'c': # C
+                importantData = ['start', 'end', 'count']
+            elif 'cfa' in rType: # CFA
+                importantData = ['start', 'end', 'count', 'surface area', 'flat area']
+            elif 'endo' in rType: # Endosome
+                importantData = ['start', 'end', 'count']
+            elif rType[0:3] == 'ser': # SER #===
+                importantData = ['start', 'end', 'count']
+            elif rType[0:2] == 'sp': # Spine #===
+                importantData = ['start', 'end', 'count', 'surface area', 'flat area', 'volume']
+            elif rType[0:2] == 'ax': # Axon #===
+                importantData = ['start', 'end', 'count']
+            else:
+                importantData = ['start', 'end', 'count']
         else:
             importantData = ['start', 'end', 'count']
         self.getData( importantData )
@@ -526,26 +554,34 @@ class rObject:
              
     def getrType(self):
         '''Returns type of character'''
-        return str(self.name[3:self.name.rfind(self.protrusion)])
+        if self.protrusion != None:
+            return self.name[re.compile(self.dendrite).match(self.name).end():re.compile(self.protrusion).search(self.name).start()+1]
+        else:
+            return None
         
     def findChildren(self):
         '''Finds children of this protrusion and puts in self.children dict under trace type'''
         children = {}
-        child_exp = re.compile(self.dendrite+'.{0,}'+self.protrusion)
+        child_exp = re.compile(self.dendrite+'.{0,}'+self.protrusion[1:])
+        dend_exp = re.compile(self.dendrite)
         for child in self.series.getObjectLists()[2]:
             if child_exp.match(child) != None:
+                # Extract from name what is in between dend and prot
+                endOfDendrite = dend_exp.match(child).end()
+                beginOfProtrusion = child.rfind(self.protrusion[1:])
                 try: # Try to add to existing entry in dictionary
-                    children[str(child[3:child.rfind(self.protrusion)])].append(child)
+                    children[str(child[endOfDendrite:beginOfProtrusion])].append(child)
                 except: # Make entry and then add to it
-                    children[str(child[3:child.rfind(self.protrusion)])] = []
-                    children[str(child[3:child.rfind(self.protrusion)])].append(child)
+                    children[str(child[endOfDendrite:beginOfProtrusion])] = []
+                    children[str(child[endOfDendrite:beginOfProtrusion])].append(child)
+                    
         return children
     
     def getSpacing(self):
         '''Returns number of spaces to add after excel sheet'''
         try:
             return max([len(self.children[child]) for child in self.children])-1
-        except: #===
+        except:
             return 0
         
     def chkSeries(self, series):
@@ -814,11 +850,11 @@ class Series:
             return self.getStartEndCount(object_name)[1]
         elif string == 'count':    
             return self.getStartEndCount(object_name)[2]
-    def getObjectLists(self):
+    def getObjectLists(self): #=== added ',' to {2}
         '''Returns sorted lists of dendrite names, protrusion names, trace names, and a list of other objects in series'''
-        dendrite_expression = 'd[0-9]{2}' # represents base dendrite name (d##)
-        protrusion_expression = 'd[0-9]{2}p[0-9]{2}$' # represents base protrusion name (d##p##)
-        trace_expression = 'd[0-9]{2}.{1,}[0-9]{2}' # represents trace name (d##<tracetype>##)
+        dendrite_expression = 'd[0-9]{2,}' # represents base dendrite name (d##)
+        protrusion_expression = 'd[0-9]{2,}p[0-9]{2,}$' # represents base protrusion name (d##p##)
+        trace_expression = 'd[0-9]{2,}.{1,}[0-9]{2,}' # represents trace name (d##<tracetype>##)
         
         # Convert expressions to usable regular expressions
         dendrite_expression = re.compile(dendrite_expression)
@@ -834,11 +870,11 @@ class Series:
             for contour in section.contours:
                 # Dendrite
                 if dendrite_expression.match(contour.name) != None:
-                    dendrites.append(contour.name[0:3])
+                    dendrites.append(contour.name[0:dendrite_expression.match(contour.name).end()]) #===
                 # Protrusion
                 if protrusion_expression.match(contour.name) != None:
                     protrusions.append(contour.name)
-                # Trace
+                # Trace === expand to > 2 digits!
                 if (trace_expression.match(contour.name) != None and
                     protrusion_expression.match(contour.name) == None):
                     traces.append(contour.name)
